"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.factoryParameters = exports.useBatchQuery = exports.useStore = exports.createGeschichte = exports.StoreContext = exports.DEFAULT_NAMESPACE = void 0;
const immer_1 = require("immer");
const react_1 = require("react");
const middleware_1 = require("zustand/middleware");
const traditional_1 = require("zustand/traditional");
const shallow_1 = require("zustand/shallow");
const middleware_js_1 = require("./middleware.js");
const utils_js_1 = require("./utils.js");
(0, immer_1.enablePatches)();
exports.DEFAULT_NAMESPACE = 'default';
exports.StoreContext = (0, react_1.createContext)(null);
const createGeschichte = (historyInstance) => {
    const thisStore = (0, middleware_js_1.converter)(historyInstance);
    const storeWithHistory = (0, middleware_js_1.historyManagement)(historyInstance)(thisStore);
    const middleware = (0, middleware_js_1.immerWithPatches)(storeWithHistory);
    if (process.env.NODE_ENV !== 'production' && typeof window !== 'undefined') {
        return (0, traditional_1.createWithEqualityFn)((0, middleware_1.subscribeWithSelector)((0, middleware_1.devtools)(middleware, { name: 'geschichte' })), shallow_1.shallow);
    }
    return (0, traditional_1.createWithEqualityFn)((0, middleware_1.subscribeWithSelector)(middleware), shallow_1.shallow);
};
exports.createGeschichte = createGeschichte;
const assertContextExists = (value) => {
    if (process.env.NODE_ENV !== 'production' && value === null) {
        throw new Error('Cannot find `GeschichteProvider` in React tree context. Please provide outer <GeschichteProvider />.');
    }
    return value;
};
const useStore = () => {
    return assertContextExists((0, react_1.useContext)(exports.StoreContext));
};
exports.useStore = useStore;
const useBatchQuery = () => {
    const store = (0, exports.useStore)();
    return store(({ batchPushState, batchReplaceState }) => ({
        batchPushState,
        batchReplaceState,
    }));
};
exports.useBatchQuery = useBatchQuery;
const factoryParameters = (config, defaultInitialValues = {}, ns = exports.DEFAULT_NAMESPACE) => {
    const flatConfig = (0, utils_js_1.flattenConfig)(config);
    const createInitialValues = (d) => typeof d === 'function' ? d() : d;
    const initBlank = (initialQueries, initialValues) => {
        // thisValues will be mutated by applyFlatConfigToState, that's why we init it with a copy of
        // the initial state.
        let thisQuery = {};
        // We produce a new state here instead of mutating defaultInitialValues.
        // Otherwise it's possible that it get's reused across executions and that will yield to readonly errors.
        const values = (0, immer_1.produce)(initialValues, (draft) => {
            thisQuery = (0, utils_js_1.applyFlatConfigToState)(flatConfig, initialQueries, ns, draft, initialValues);
        });
        return {
            initialValues,
            query: thisQuery,
            values,
        };
    };
    const useQuery = () => {
        const useStore = assertContextExists((0, react_1.useContext)(exports.StoreContext));
        const { register, pushState, replaceState, resetPush, resetReplace, initialQueries, } = useStore(({ register, pushState, replaceState, resetPush, resetReplace, initialQueries, }) => ({
            initialQueries,
            pushState,
            register,
            replaceState,
            resetPush,
            resetReplace,
        }), shallow_1.shallow);
        const initialRegisterState = (0, react_1.useMemo)(() => {
            const initialValues = createInitialValues(defaultInitialValues);
            return initBlank(initialQueries(), initialValues);
            // eslint-disable-next-line react-hooks/exhaustive-deps
        }, [useStore, defaultInitialValues]);
        const [currentState, setCurrentState] = (0, react_1.useState)({
            initialValues: initialRegisterState.initialValues,
            values: initialRegisterState.values,
        });
        (0, react_1.useEffect)(() => {
            const { unsubscribe: unregister, values, initialValues, } = register(config, flatConfig, ns, initialRegisterState.initialValues, initialRegisterState.query, initialRegisterState.values);
            if (initialRegisterState.values !== values ||
                initialRegisterState.initialValues !== initialValues ||
                currentState.initialValues !== initialValues ||
                currentState.values !== values) {
                setCurrentState({ values, initialValues });
            }
            const unsubscribe = useStore.subscribe((state) => state.namespaces[ns] && {
                initialValues: state.namespaces[ns].initialValues,
                values: state.namespaces[ns].values,
            }, (state) => {
                if (state) {
                    setCurrentState(state);
                }
            }, { equalityFn: shallow_1.shallow });
            return () => {
                unsubscribe();
                unregister();
            };
            // eslint-disable-next-line react-hooks/exhaustive-deps
        }, [initialRegisterState]);
        const values = currentState.values;
        const initialValues = currentState.initialValues;
        const createQuery = (0, react_1.useCallback)((values) => {
            return (0, utils_js_1.createQueryObject)(flatConfig, ns, values, initialValues);
        }, [initialValues]);
        return (0, react_1.useMemo)(() => ({
            createQuery: (customValues) => createQuery(customValues || values),
            createQueryString: (customValues) => new URLSearchParams(createQuery(customValues || values)).toString(),
            initialValues,
            pushState: (state, options) => pushState(ns, state, options),
            replaceState: (state, options) => replaceState(ns, state, options),
            resetPush: () => resetPush(ns),
            resetReplace: () => resetReplace(ns),
            values,
        }), [
            values,
            initialValues,
            pushState,
            replaceState,
            resetPush,
            resetReplace,
            createQuery,
        ]);
    };
    const createQueryString = (values, initialValues) => {
        const thisInitialValues = typeof initialValues === 'undefined'
            ? createInitialValues(defaultInitialValues)
            : initialValues;
        return new URLSearchParams((0, utils_js_1.createQueryObject)(flatConfig, ns, values, thisInitialValues)).toString();
    };
    const parseQueryString = (query, initialValues) => {
        const thisInitialValues = typeof initialValues === 'undefined'
            ? createInitialValues(defaultInitialValues)
            : initialValues;
        return (0, immer_1.produce)({}, (draft) => {
            const parsedQuery = new URLSearchParams(query);
            (0, utils_js_1.applyFlatConfigToState)(flatConfig, Object.fromEntries(parsedQuery.entries()), ns, draft, thisInitialValues);
        });
    };
    return { useQuery, createQueryString, parseQueryString };
};
exports.factoryParameters = factoryParameters;
